package org.processmining.acceptingpetrinet.algorithms;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.acceptingpetrinet.parameters.MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.semantics.petrinet.Marking;

public class MergeAcceptingPetriNetArrayIntoAcceptingPetriNetAlgorithm {

	public AcceptingPetriNet apply(PluginContext context, AcceptingPetriNetArray nets,
			MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters parameters) {
		AcceptingPetriNet mergedNet = AcceptingPetriNetFactory.createAcceptingPetriNet();
		mergedNet.init(PetrinetFactory.newPetrinet(nets.getSize() > 0 ? nets.getNet(0).getNet().getLabel()
				: "Empty net"));
		Marking initialMarking = mergedNet.getInitialMarking();
		mergedNet.getFinalMarkings().add(new Marking());
		Map<Place, Place> placeMap = new HashMap<Place, Place>();
		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>();
		Map<String, Transition> transitionLabelMap = new HashMap<String, Transition>();
		for (int index = 0; index < nets.getSize(); index++) {
			Petrinet net = nets.getNet(index).getNet();
			for (Transition transition : net.getTransitions()) {
				Transition mergedTransition = null;
				if (transition.isInvisible()) {
					mergedTransition = mergedNet.getNet().addTransition(transition.getLabel());
					mergedTransition.setInvisible(true);
				} else {
					String label = transition.getLabel();
					mergedTransition = transitionLabelMap.get(label);
					if (mergedTransition == null) {
						mergedTransition = mergedNet.getNet().addTransition(label);
						mergedTransition.setInvisible(false);
						transitionLabelMap.put(label, mergedTransition);
					}
				}
				transitionMap.put(transition, mergedTransition);
			}
			for (Place place : net.getPlaces()) {
				Place mergedPlace = mergedNet.getNet().addPlace(place.getLabel());
				placeMap.put(place, mergedPlace);
				if (nets.getNet(index).getInitialMarking().contains(place)) {
					initialMarking.add(mergedPlace, nets.getNet(index).getInitialMarking().occurrences(place));
				}
			}
			//			if (!nets.getNet(index).getFinalMarkings().isEmpty()) {
			Set<Marking> extendedFinalMarkings = new HashSet<Marking>();
			//				System.out.println("New Final markings");
			for (Marking currentFinalMarking : mergedNet.getFinalMarkings()) {
				for (Marking finalMarking : nets.getNet(index).getFinalMarkings()) {
					Marking marking = new Marking(currentFinalMarking);
					for (Place p : finalMarking.baseSet()) {
						marking.add(placeMap.get(p), finalMarking.occurrences(p));
					}
					extendedFinalMarkings.add(marking);
					//						System.out.println(marking);
				}
//				Marking marking = new Marking(currentFinalMarking);
				//				Marking finalMarking = nets.getNet(index).getInitialMarking();
				//				for (Place p : finalMarking.baseSet()) {
				//					marking.add(placeMap.get(p), finalMarking.occurrences(p));
				//				}
//				extendedFinalMarkings.add(marking);
				//					System.out.println(marking);
			}
			//				if (extendedFinalMarkings.isEmpty()) {
			//					extendedFinalMarkings.add(new Marking());
			//				}
			mergedNet.setFinalMarkings(extendedFinalMarkings);
			//			}
			for (PetrinetEdge<?, ?> edge : net.getEdges()) {
				if (edge instanceof Arc) {
					Arc arc = (Arc) edge;
					if (arc.getSource() instanceof Place) {
						mergedNet.getNet().addArc(placeMap.get(arc.getSource()), transitionMap.get(arc.getTarget()));
					} else {
						mergedNet.getNet().addArc(transitionMap.get(arc.getSource()), placeMap.get(arc.getTarget()));
					}
				}
			}
		}
		if (parameters.isAddWFPlaces()) {
			Place initialPlace = null;
			Place finalPlace = null;
			for (Transition transition : mergedNet.getNet().getTransitions()) {
				if (mergedNet.getNet().getInEdges(transition).isEmpty()) {
					if (initialPlace == null) {
						initialPlace = mergedNet.getNet().addPlace("i");
						mergedNet.getInitialMarking().add(initialPlace);
					}
					mergedNet.getNet().addArc(initialPlace, transition);
				}
				if (mergedNet.getNet().getOutEdges(transition).isEmpty()) {
					if (finalPlace == null) {
						finalPlace = mergedNet.getNet().addPlace("o");
						Marking finalMarking = new Marking();
						finalMarking.add(finalPlace);
						mergedNet.getFinalMarkings().add(finalMarking);
					}
					mergedNet.getNet().addArc(transition, finalPlace);
				}
			}
			if (initialPlace != null && finalPlace != null) {
				if (mergedNet.getNet().getOutEdges(initialPlace).size() > 1) {
					Transition transition = mergedNet.getNet().addTransition("redo");
					transition.setInvisible(true);
					mergedNet.getNet().addArc(finalPlace, transition);
					mergedNet.getNet().addArc(transition, initialPlace);
				}
			}
		}
		return mergedNet;
	}
}
