package org.processmining.acceptingpetrinet.plugins;

import java.util.Collection;

import org.processmining.acceptingpetrinet.algorithms.PackPetrinetAlgorithm;
import org.processmining.acceptingpetrinet.connections.PackPetrinetConnection;
import org.processmining.acceptingpetrinet.help.PackPetrinetHelp;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.parameters.PackPetrinetParameters;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;

@Plugin(name = "Pack Petri net", parameterLabels = { "Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class }, help = PackPetrinetHelp.TEXT)
public class PackPetrinetPlugin extends PackPetrinetAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Pack Accepting Petri net, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet runDefault(PluginContext context, Petrinet net) {
		PackPetrinetParameters parameters = new PackPetrinetParameters();
		return runConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public AcceptingPetriNet packDefault(PluginContext context, Petrinet net) {
		return runDefault(context, net);
	}

	@PluginVariant(variantLabel = "Pack Accepting Petri net, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet run(PluginContext context, Petrinet net, PackPetrinetParameters parameters) {
		return runConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public AcceptingPetriNet packParameters(PluginContext context, Petrinet net, PackPetrinetParameters parameters) {
		return run(context, net, parameters);
	}

	private AcceptingPetriNet runConnection(PluginContext context, Petrinet net,
			PackPetrinetParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<PackPetrinetConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(PackPetrinetConnection.class, context, net);
				for (PackPetrinetConnection connection : connections) {
					if (connection.getObjectWithRole(PackPetrinetConnection.NET).equals(net)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(PackPetrinetConnection.APN);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		AcceptingPetriNet apn = apply(context, net, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(new PackPetrinetConnection(apn, net, parameters));
		}
		return apn;
	}

}
