package org.processmining.acceptingpetrinet.plugins;

import java.util.Collection;

import org.processmining.acceptingpetrinet.algorithms.RemoveStartEndTransitionsAlgorithm;
import org.processmining.acceptingpetrinet.connections.RemoveStartEndTransitionsConnection;
import org.processmining.acceptingpetrinet.help.RemoveStartEndTransitionsHelp;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.parameters.RemoveStartEndTransitionsParameters;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.semantics.petrinet.Marking;

@Plugin(name = "Remove Start and End Transitions", parameterLabels = { "Accepting Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class }, help = RemoveStartEndTransitionsHelp.TEXT)
public class RemoveStartEndTransitionsPlugin extends RemoveStartEndTransitionsAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Remove Start and End Transitions, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet runDefault(PluginContext context, AcceptingPetriNet net) {
		RemoveStartEndTransitionsParameters parameters = new RemoveStartEndTransitionsParameters();
		return runConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public AcceptingPetriNet reduceDefault(PluginContext context, AcceptingPetriNet net) {
		return runDefault(context, net);
	}

	@PluginVariant(variantLabel = "Remove Start and End Transitions, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet run(PluginContext context, AcceptingPetriNet net,
			RemoveStartEndTransitionsParameters parameters) {
		return runConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public AcceptingPetriNet reduceParameters(PluginContext context, AcceptingPetriNet net,
			RemoveStartEndTransitionsParameters parameters) {
		return run(context, net, parameters);
	}

	private AcceptingPetriNet runConnection(PluginContext context, AcceptingPetriNet net,
			RemoveStartEndTransitionsParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<RemoveStartEndTransitionsConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(RemoveStartEndTransitionsConnection.class,
						context, net);
				for (RemoveStartEndTransitionsConnection connection : connections) {
					if (connection.getObjectWithRole(RemoveStartEndTransitionsConnection.NET).equals(net)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(RemoveStartEndTransitionsConnection.REDUCEDNET);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		AcceptingPetriNet reducedNet = apply(context, net, parameters);
		if (parameters.isTryConnections()) {
			context.getProvidedObjectManager().createProvidedObject(
					"Initial marking for " + reducedNet.getNet().getLabel(), reducedNet.getInitialMarking(),
					Marking.class, context);
			context.addConnection(new InitialMarkingConnection(reducedNet.getNet(), reducedNet.getInitialMarking()));
			context.getConnectionManager().addConnection(
					new RemoveStartEndTransitionsConnection(net, reducedNet, parameters));
		}
		return reducedNet;
	}

}
