package org.processmining.acceptingpetrinet.plugins;

import java.util.Collection;

import org.processmining.acceptingpetrinet.algorithms.UndoubleAcceptingPetriNetAlgorithm;
import org.processmining.acceptingpetrinet.connections.UndoubleAcceptingPetriNetConnection;
import org.processmining.acceptingpetrinet.help.UndoubleAcceptingPetriNetHelp;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.parameters.UndoubleAcceptingPetriNetParameters;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.semantics.petrinet.Marking;

@Plugin(name = "Undouble Accepting Petri net", parameterLabels = { "Accepting Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class }, help = UndoubleAcceptingPetriNetHelp.TEXT)
public class UndoubleAcceptingPetriNetPlugin extends UndoubleAcceptingPetriNetAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Undouble Accepting Petri net, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet runDefault(PluginContext context, AcceptingPetriNet apn) {
		UndoubleAcceptingPetriNetParameters parameters = new UndoubleAcceptingPetriNetParameters();
		return runConnection(context, apn, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public AcceptingPetriNet undoubleDefault(PluginContext context, AcceptingPetriNet apn) {
		return runDefault(context, apn);
	}

	@PluginVariant(variantLabel = "Unpack Accepting Petri net, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet run(PluginContext context, AcceptingPetriNet apn, UndoubleAcceptingPetriNetParameters parameters) {
		return runConnection(context, apn, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public AcceptingPetriNet undoubleParameters(PluginContext context, AcceptingPetriNet apn, UndoubleAcceptingPetriNetParameters parameters) {
		return run(context, apn, parameters);
	}

	private AcceptingPetriNet runConnection(PluginContext context, AcceptingPetriNet net,
			UndoubleAcceptingPetriNetParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<UndoubleAcceptingPetriNetConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(UndoubleAcceptingPetriNetConnection.class,
						context, net);
				for (UndoubleAcceptingPetriNetConnection connection : connections) {
					if (connection.getObjectWithRole(UndoubleAcceptingPetriNetConnection.NET).equals(net)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(UndoubleAcceptingPetriNetConnection.REDUCEDNET);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		AcceptingPetriNet undoubledNet = apply(context, net, parameters);
		if (parameters.isTryConnections()) {
			context.getProvidedObjectManager().createProvidedObject(
					"Initial marking for " + undoubledNet.getNet().getLabel(), undoubledNet.getInitialMarking(),
					Marking.class, context);
			context.addConnection(new InitialMarkingConnection(undoubledNet.getNet(), undoubledNet.getInitialMarking()));
			context.getConnectionManager().addConnection(
					new UndoubleAcceptingPetriNetConnection(net, undoubledNet, parameters));
		}
		return undoubledNet;
	}
}
